-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftIsPetData")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Save Pet Data"] = true,
	["Stores pet information between sessions"] = true,
} end)

-----------------------------------------------------------------------------

local petNames = { }

-----------------------------------------------------------------------------

local AloftIsPetData = Aloft:NewModule("IsPetData", "AceEvent-2.0")
AloftIsPetData.db = Aloft:AcquireDBNamespace("isPetData")
Aloft:RegisterDefaults("isPetData", "server", { petNames = { } })
Aloft:RegisterDefaults("isPetData", "profile", { save = false })

-----------------------------------------------------------------------------

Aloft.Options.args.data.args.pet =
{
	type = 'toggle',
	name = L["Save Pet Data"],
	desc = L["Stores pet information between sessions"],
	get = function() return AloftIsPetData.db.profile.save end,
	set = function(v)
			AloftIsPetData.db.profile.save = v
			if v then
				AloftIsPetData.db.server.petNames = petNames
			else
				AloftIsPetData.db.server.petNames = { }
			end
		  end,
}

-----------------------------------------------------------------------------

AloftTagDirectAccessData.isPet = true
AloftTagData.isPet = { data = "isPet", events = "Aloft:OnIsPetDataChanged" }

function AloftIsPetData:ProvidesData()
	return "isPet"
end

function AloftIsPetData:EnableDataSource()
	-- Events to update our class database
    self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "UpdatePetFromMouseOver")
    self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdatePetFromTarget")
end

-----------------------------------------------------------------------------

function AloftIsPetData:OnEnable()
	if self.db.profile.save then
		petNames = self.db.server.petNames
	end
end

-----------------------------------------------------------------------------

function AloftIsPetData:UpdateData(aloftData)
	aloftData.isPet = nil
	
	-- This is so that pets named the same as players we've encountered aren't given a class incorrectly
	if aloftData.type == "friendlyNPC" or aloftData.type == "hostile" then
		aloftData.isPet = petNames[aloftData.name]
	end
end

-----------------------------------------------------------------------------

function AloftIsPetData:SetPet(name)
	if not name or petNames[name] then return end
	petNames[name] = true
	
	local aloftData = Aloft:GetNameplate(name)
	if aloftData then
		if not aloftData.isPet and (aloftData.type == "friendlyNPC" or aloftData.type == "hostile") then
			aloftData.isPet = true
			self:TriggerEvent("Aloft:OnIsPetDataChanged", aloftData)
		end
	end
end

-----------------------------------------------------------------------------

local function IsPet(unitid)
	return UnitExists(unitid) and not UnitIsPlayer(unitid) and UnitPlayerControlled(unitid)
end

function AloftIsPetData:UpdatePetFromMouseOver()
	if IsPet("mouseover") then
		self:SetPet(UnitName("mouseover"))
	end
end

function AloftIsPetData:UpdatePetFromTarget()
	if IsPet("target") then
		self:SetPet(UnitName("target"))
	end
end

-----------------------------------------------------------------------------
