-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCommentText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Comment Text"] = true,
	["Comment text options"] = true,
	["Enable"] = true,
	["Show comments on nameplates"] = true,
	["Font"] = true,
	["Sets the font for comment text"] = true,

	["Set Comment"] = true,
	["Sets the comment for your current target"] = true,
	["<Any String>"] = true,

	["Typeface"] = true,
	["Comment typeface options"] = true,
	["Font Size"] = true,
	["Sets the font height of the comment text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on comment text"] = true,
	["Outline"] = true,
	["Sets the outline for comment text"] = true,

	["Position"] = true,
	["Adjust comment tag position"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the comment text"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the comment text"] = true,
	["X Offset"] = true,
	["X offset of the comment text"] = true,
	["Y Offset"] = true,
	["Y offset of the comment text"] = true,
	["Alpha"] = true,
	["Sets the alpha of the comment text"] = true,
	["Color"] = true,
	["Sets the comment text color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,

	["NoSetComment"] = "|cffff0000You must have a target to set a comment|r",
	["CommentAdded"] = "Added |cff00ff00%s|r -> |cff00ff00%s|r",

	["Advanced"] = true,
	["Options for expert users"] = true,
	["Format"] = true,
	["Comment tag"] = true,
	["<Any tag string>"] = true,
} end)

-----------------------------------------------------------------------------

local AloftCommentText = Aloft:NewModule("CommentText", "AceEvent-2.0")
AloftCommentText.db = Aloft:AcquireDBNamespace("commentText")
Aloft:RegisterDefaults("commentText", "profile", {
	enable			= false,
	alpha			= 1,
	point			= "CENTER",
	relativeToPoint	= "CENTER",
	offsetX			= 0,
	offsetY			= -10,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= false,
	outline			= "OUTLINE",
	format			= "[comment]",
	color			= { 1, 1, 1 }
})

Aloft.Options.args.comment =
{
	type = 'group',
	name = L["Comment Text"],
	desc = L["Comment text options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show comments on nameplates"],
			get = function() return AloftCommentText.db.profile.enable end,
			set = function(v) AloftCommentText.db.profile.enable = v AloftCommentText:UpdateTag() AloftCommentText:UpdateAll() end,
			order = 1
		},
		setComment =
		{
			type = 'text',
			name = L["Set Comment"],
			desc = L["Sets the comment for your current target"],
			usage = L["<Any String>"],
			disabled = function() return not AloftCommentText.db.profile.enable end,
			get = function() return "" end,
			set = function(v) 
					local setFor = Aloft:GetModule("CommentData"):SetCommentForTarget(v)
					if setFor then
						Aloft:Print(string.format(L["CommentAdded"], setFor, v))
					else
						Aloft:Print(L["NoSetComment"])
					end
				  end,
			order = 2
		},
		typeface =
		{
			type = "group",
			name = L["Typeface"],
			desc = L["Comment typeface options"],
			disabled = function() return not AloftCommentText.db.profile.enable end,
			args =
			{
				font =
				{
					type = "text", 
					name = L["Font"], 
					desc = L["Sets the font for comment text"],
					get = function() return AloftCommentText.db.profile.font end,
					set = function(v) AloftCommentText.db.profile.font = v AloftCommentText:UpdateAll() end,
					validate = SML_font,
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the comment text"],
					max = 16,
					min = 5,
					step = 1,
					get = function() return AloftCommentText.db.profile.fontSize end,
					set = function(value) AloftCommentText.db.profile.fontSize = value AloftCommentText:UpdateAll() end
				},
				shadow =
				{
					type = 'toggle',
					name = L["Font Shadow"],
					desc = L["Show font shadow on comment text"],
					get = function() return AloftCommentText.db.profile.shadow end,
					set = function(v) AloftCommentText.db.profile.shadow = v AloftCommentText:UpdateAll() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the outline for comment text"],
					get = function() return AloftCommentText.db.profile.outline end,
					set = function(value) AloftCommentText.db.profile.outline = value AloftCommentText:UpdateAll() end,
					validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
				},
			},
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust comment tag position"],
			disabled = function() return not AloftCommentText.db.profile.enable end,
			args =
			{
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the comment text"],
					get = function() return AloftCommentText.db.profile.point end,
					set = function(v) AloftCommentText.db.profile.point = v AloftCommentText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the comment text"],
					get = function() return AloftCommentText.db.profile.relativeToPoint end,
					set = function(v) AloftCommentText.db.profile.relativeToPoint = v AloftCommentText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the comment text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCommentText.db.profile.offsetX end,
					set = function(v) AloftCommentText.db.profile.offsetX = v AloftCommentText:UpdateAll() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the comment text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCommentText.db.profile.offsetY end,
					set = function(v) AloftCommentText.db.profile.offsetY = v AloftCommentText:UpdateAll() end
				},
			},
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the alpha of the comment text"],
			min = 0,
			max = 1.0,
			step = 0.05,
			disabled = function() return not AloftCommentText.db.profile.enable end,
			get = function() return AloftCommentText.db.profile.alpha end,
			set = function(v) AloftCommentText.db.profile.alpha = v AloftCommentText:UpdateAll() end
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the comment text color"],
			disabled = function() return not AloftCommentText.db.profile.enable end,
			get = function() return unpack(AloftCommentText.db.profile.color) end,
			set = function(r, g, b, a) AloftCommentText.db.profile.color = { r, g, b, a } AloftCommentText:UpdateAll() end,
		},
		advanced  =
		{
			type = 'group',
			name = L["Advanced"],
			desc = L["Options for expert users"],
			order = -1,
			disabled = function() return not AloftCommentText.db.profile.enable end,
			args =
			{
				format = 
				{
					type = 'text',
					name = L["Format"],
					desc = L["Comment tag"],
					usage = L["<Any tag string>"],
					get = function() return AloftCommentText.db.profile.format end,
					set = function(v) AloftCommentText.db.profile.format = v Aloft:DetermineDataSources() AloftCommentText:UpdateAll() end,
				}
			},
		},
	},
}

-----------------------------------------------------------------------------

local textMethod = nil
local textMethodData = { }

local textRegionPool = { }

-----------------------------------------------------------------------------

function AloftCommentText:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		if self.db.profile.enable then
			if aloftData.commentRegion then
				self:PrepareText(aloftData.commentRegion, self.db.profile)
				self:PlaceFrame(aloftData.commentRegion, aloftData.healthBar, self.db.profile)
			end
			if aloftData:IsShown() then
				self:UpdateText(aloftData)
			end
		else
			self:OnNameplateHide(aloftData)
		end
	end
	for textRegion in pairs(textRegionPool) do
		self:PrepareText(textRegion, self.db.profile)
	end
end

-----------------------------------------------------------------------------

function AloftCommentText:UpdateTag()
	if self.db.profile.enable then 
		self.db.profile.format = "[comment]"
	else
		self.db.profile.format = ""
	end

	Aloft:DetermineDataSources()
end

-----------------------------------------------------------------------------

function AloftCommentText:RegisterEvents()
	local profile = self.db.profile
	self:UnregisterAllEvents()

	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)
	
	if profile.enable then
		self:RegisterEvent("Aloft:OnNameplateShow", "UpdateText")
		self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")

		for events in pairs(textMethodData.events) do
			self:RegisterEvent(events, "UpdateText")
		end
	end
end

local dataRequiredList = { }
function AloftCommentText:RequiresData()
	local profile = self.db.profile

	textMethodData = AloftTag:CreateTag(profile.format)
	textMethod = textMethodData.method

	self:RegisterEvents()

	if not profile.enable then
		return
	end

	for i=1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	for data in pairs(textMethodData.data) do
		table.insert(dataRequiredList, data)
	end

	return unpack(dataRequiredList)
end

function AloftCommentText:OnEnable()
	self:UpdateAll()
end

function AloftCommentText:OnDisable()
	self:UnregisterAllEvents()
	
	for aloftData in Aloft:IterateNameplates() do
		local commentRegion = aloftData.commentRegion
		if commentRegion then
			commentRegion:Hide()
		end
	end
end

function AloftCommentText:UpdateText(aloftData)
	local text = textMethod(aloftData)

	if text ~= "" then
		-- Check if this nameplate already has a text region assigned to it
		local commentRegion = aloftData.commentRegion
		if not commentRegion then
			commentRegion = next(textRegionPool)
			if commentRegion then
				-- We have a recycled text region. Text format already setup, just need to place it
				textRegionPool[commentRegion] = nil
				commentRegion:SetParent(aloftData.nameplateFrame)
				commentRegion:Show()
			else
				-- We need a new text region.
				commentRegion = aloftData:CreateFontString()
				self:PrepareText(commentRegion, self.db.profile)
			end
			self:PlaceFrame(commentRegion, aloftData.healthBar, self.db.profile)
			aloftData.commentRegion = commentRegion
		end
		commentRegion:SetText(text)
	else
		self:OnNameplateHide(aloftData)
	end
end

function AloftCommentText:OnNameplateHide(aloftData)
	local commentRegion = aloftData.commentRegion
	if commentRegion then
		commentRegion:Hide()
		textRegionPool[commentRegion] = true
		aloftData.commentRegion = nil
	end
end

-----------------------------------------------------------------------------
