-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftHealthText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Health Text"] = true,
	["Health text options"] = true,
	["Enable"] = true,
	["Show health text on nameplates"] = true,

	["Typeface"] = true,
	["Health text typeface options"] = true,
	["Font"] = true,
	["Sets the font for health text"] = true,
	["Font Size"] = true,
	["Sets the font height of the health text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on health text"] = true,
	["Outline"] = true,
	["Sets the outline for health text"] = true,
	["Mode"] = true,
	["Choose what to show for health text"] = true,

	["Position"] = true,
	["Adjust health text position"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the health text"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the health text"] = true,
	["X Offset"] = true,
	["X offset of the health text"] = true,
	["Y Offset"] = true,
	["Y offset of the health text"] = true,
	["Alpha"] = true,
	["Sets the alpha of the health text"] = true,
	["Color"] = true,
	["Sets the health text color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
	
	["Percent"] = true,
	["Group Health"] = true,
	["Group Health Deficit"] = true,
	["Health"] = true,
	["Full"] = true,
	["Smart"] = true,

	["Advanced"] = true,
	["Options for expert users"] = true,
	["Format"] = true,
	["Health tag"] = true,
	["<Any tag string>"] = true,
} end)

-----------------------------------------------------------------------------

local healthTags =
{
	NONE				= "",
	PERCENT				= "[healthPercent:percent]",
	GROUPHEALTH 		= "[isGroup:healthBarValue:short]",
	GROUPHEALTHDEFICIT	= "[isGroup:healthBarDeficit:hideZero:negate:short]",
	HEALTH				= "[health:short][healthPercent:percent:hideKnownHealth]",
	FULL				= "[health:short:suffix(\"/\")][maxHealth:short:suffix(\" (\")][healthPercent:percent][health:\")\"]",
	SMART				= "[not(isFriendly):health:short][not(isMaxHealth):healthPercent:hideKnownHealth:percent][isFriendly:healthDeficit:negate:hideZero:short:hexColor(\"ff8080\")]",
}

-----------------------------------------------------------------------------

local AloftHealthText = Aloft:NewModule("HealthText", "AceEvent-2.0")
AloftHealthText.db = Aloft:AcquireDBNamespace("healthText")
Aloft:RegisterDefaults("healthText", "profile", {
	enable			= false,
	alpha			= 1,
	point			= "RIGHT",
	relativeToPoint	= "RIGHT",
	offsetX			= 0,
	offsetY			= 0,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= false,
	outline			= "OUTLINE",
	mode			= "PERCENT",
	format			= "",
	color			= { 1, 1, 1 }
})

Aloft.Options.args.healthBar.args.healthText =
{
	type = 'group',
	name = L["Health Text"],
	desc = L["Health text options"],
	disabled = function() return not Aloft:IsActive() end,
	order = -1,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show health text on nameplates"],
			get = function() return AloftHealthText.db.profile.enable end,
			set = function(v) AloftHealthText.db.profile.enable = v AloftHealthText:UpdateTag() AloftHealthText:UpdateAll() end,
			order = 1
		},
		typeface =
		{
			type = "group",
			name = L["Typeface"],
			desc = L["Health text typeface options"],
			disabled = function() return not AloftHealthText.db.profile.enable end,
			args =
			{
				font =
				{
					type = "text", 
					name = L["Font"], 
					desc = L["Sets the font for health text"],
					get = function() return AloftHealthText.db.profile.font end,
					set = function(v) AloftHealthText.db.profile.font = v AloftHealthText:UpdateAll() end,
					validate = SML_font,
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the health text"],
					max = 16,
					min = 5,
					step = 1,
					get = function() return AloftHealthText.db.profile.fontSize end,
					set = function(value) AloftHealthText.db.profile.fontSize = value AloftHealthText:UpdateAll() end
				},
				shadow =
				{
					type = 'toggle',
					name = L["Font Shadow"],
					desc = L["Show font shadow on health text"],
					get = function() return AloftHealthText.db.profile.shadow end,
					set = function(v) AloftHealthText.db.profile.shadow = v AloftHealthText:UpdateAll() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the outline for health text"],
					get = function() return AloftHealthText.db.profile.outline end,
					set = function(value) AloftHealthText.db.profile.outline = value AloftHealthText:UpdateAll() end,
					validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
				},
			},
		},
		mode =
		{
			type = 'text',
			name = L["Mode"],
			desc = L["Choose what to show for health text"],
			disabled = function() return not AloftHealthText.db.profile.enable end,
			get = function() return AloftHealthText.db.profile.mode end,
			set = function(value) AloftHealthText.db.profile.mode = value AloftHealthText:UpdateTag() AloftHealthText:UpdateAll() end,
			validate = { PERCENT = L["Percent"], GROUPHEALTH = L["Group Health"], GROUPHEALTHDEFICIT = L["Group Health Deficit"], HEALTH = L["Health"], FULL = L["Full"], SMART = L["Smart"] },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust health text position"],
			disabled = function() return not AloftHealthText.db.profile.enable end,
			args =
			{
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the health text"],
					get = function() return AloftHealthText.db.profile.point end,
					set = function(v) AloftHealthText.db.profile.point = v AloftHealthText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the health text"],
					get = function() return AloftHealthText.db.profile.relativeToPoint end,
					set = function(v) AloftHealthText.db.profile.relativeToPoint = v AloftHealthText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the health text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftHealthText.db.profile.offsetX end,
					set = function(v) AloftHealthText.db.profile.offsetX = v AloftHealthText:UpdateAll() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the health text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftHealthText.db.profile.offsetY end,
					set = function(v) AloftHealthText.db.profile.offsetY = v AloftHealthText:UpdateAll() end
				},
			},
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the alpha of the health text"],
			min = 0,
			max = 1.0,
			step = 0.05,
			disabled = function() return not AloftHealthText.db.profile.enable end,
			get = function() return AloftHealthText.db.profile.alpha end,
			set = function(v) AloftHealthText.db.profile.alpha = v AloftHealthText:UpdateAll() end
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the health text color"],
			disabled = function() return not AloftHealthText.db.profile.enable end,
			get = function() return unpack(AloftHealthText.db.profile.color) end,
			set = function(r, g, b, a) AloftHealthText.db.profile.color = { r, g, b, a } AloftHealthText:UpdateAll() end,
		},
		advanced  =
		{
			type = 'group',
			name = L["Advanced"],
			desc = L["Options for expert users"],
			order = -1,
			disabled = function() return not AloftHealthText.db.profile.enable end,
			args =
			{
				format = 
				{
					type = 'text',
					name = L["Format"],
					desc = L["Health tag"],
					usage = L["<Any tag string>"],
					get = function() return AloftHealthText.db.profile.format end,
					set = function(v) AloftHealthText.db.profile.format = v Aloft:DetermineDataSources() AloftHealthText:UpdateAll() end,
				}
			},
		},
	},
}

-----------------------------------------------------------------------------

local textMethod = nil
local textMethodData = { }
local textRegionPool = { }

-----------------------------------------------------------------------------

function AloftHealthText:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		if self.db.profile.enable then
			if aloftData.healthTextRegion then
				self:PrepareText(aloftData.healthTextRegion, self.db.profile)
				self:PlaceFrame(aloftData.healthTextRegion, aloftData.healthBar, self.db.profile)
			end
			if aloftData:IsShown() then
				self:UpdateText(aloftData)
			end
		else
			self:OnNameplateHide(aloftData)
		end
	end
	for textRegion in pairs(textRegionPool) do
		self:PrepareText(textRegion, self.db.profile)
	end
end

-----------------------------------------------------------------------------

function AloftHealthText:UpdateTag()
	local tag = self.db.profile.mode
	if not self.db.profile.enable then tag = "NONE"	end
	self.db.profile.format = healthTags[tag]

	Aloft:DetermineDataSources()
end

-----------------------------------------------------------------------------

function AloftHealthText:RegisterEvents()
	local profile = self.db.profile
	self:UnregisterAllEvents()

	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)
	
	if profile.enable then
		self:RegisterEvent("Aloft:OnNameplateShow", "UpdateText")
		self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")

		for events in pairs(textMethodData.events) do
			self:RegisterEvent(events, "UpdateText")
		end
	end
end

local dataRequiredList = { }
function AloftHealthText:RequiresData()
	local profile = self.db.profile

	textMethodData = AloftTag:CreateTag(profile.format)
	textMethod = textMethodData.method

	self:RegisterEvents()

	if not profile.enable then
		return
	end

	for i=1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	for data in pairs(textMethodData.data) do
		table.insert(dataRequiredList, data)
	end

	return unpack(dataRequiredList)
end

function AloftHealthText:OnEnable()
	self:UpdateAll()
end

function AloftHealthText:OnDisable()
	self:UnregisterAllEvents()
	
	for aloftData in Aloft:IterateNameplates() do
		local healthTextRegion = aloftData.healthTextRegion
		if healthTextRegion then
			healthTextRegion:Hide()
		end
	end
end

function AloftHealthText:UpdateText(aloftData)
    local text = textMethod(aloftData)
            
    if text ~= "" then
        -- Check if this nameplate already has a text region assigned to it
        local healthTextRegion = aloftData.healthTextRegion
        if not healthTextRegion then
            healthTextRegion = next(textRegionPool)
            if healthTextRegion then
                -- We have a recycled text region. Text format already setup, just need to place it
                textRegionPool[healthTextRegion] = nil
                healthTextRegion:SetParent(aloftData.nameplateFrame)
                healthTextRegion:Show()
            else
                -- We need a new text region.
                healthTextRegion = aloftData:CreateFontString()
                self:PrepareText(healthTextRegion, self.db.profile)
            end
            self:PlaceFrame(healthTextRegion, aloftData.healthBar, self.db.profile)
            aloftData.healthTextRegion = healthTextRegion
        end
        healthTextRegion:SetText(text)
    else
        self:OnNameplateHide(aloftData)
    end
end

function AloftHealthText:OnNameplateHide(aloftData)
    local healthTextRegion = aloftData.healthTextRegion
    if healthTextRegion then
        healthTextRegion:Hide()
        textRegionPool[healthTextRegion] = true
        aloftData.healthTextRegion = nil
    end
end

-----------------------------------------------------------------------------
