-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftManaBar")
local SML = AceLibrary("SharedMedia-1.0")
local SML_statusbar = SML:List("statusbar")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Mana Bar"] = true,
	["Mana Bar options"] = true,
	["Enable"] = true,
	["Show mana bars on group member nameplates"] = true,

	["Texture"] = true,
	["Sets the mana bar texture"] = true,
	["Alpha"] = true,
	["Sets the mana bar alpha"] = true,

	["Height"] = true,
	["Sets the height of the mana bar"] = true,
	
	["Position"] = true,
	["Adjust the mana bar's position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the mana bar"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the mana bar"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the mana bar"] = true,
	
	["Colors"] = true,
	["Customize colors"] = true,
	["Backdrop Color"] = true,
	["Sets the backdrop color of the mana bar"] = true,
	["Mana Color"] = true,
	["Sets the color for mana bars"] = true,
	["Rage Color"] = true,
	["Sets the color for rage bars"] = true,
	["Focus Color"] = true,
	["Sets the color for focus bars"] = true,
	["Energy Color"] = true,
	["Sets the color for energy bars"] = true,
	["Happiness Color"] = true,
	["Sets the color for happiness bars"] = true,
	["Reset to Defaults"] = true,
	["Resets all colors to their defaults"] = true,

	["Advanced"] = true,
	["Options for expert users"] = true,
	["Color Format"] = true,
	["Color tag"] = true,
	["<Any tag string>"] = true,
} end)

-----------------------------------------------------------------------------

local AloftManaBar = Aloft:NewModule("ManaBar", "AceEvent-2.0")
AloftManaBar.db = Aloft:AcquireDBNamespace("manaBar")
Aloft:RegisterDefaults("manaBar", "profile", {
	enable			= false,
	texture			= "Blizzard",
	height			= 4,
	backdropColor	= { 0.25, 0.25, 0.25, 0.5 },
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= -1,
	},
	powerTypeColors	=
	{
        [0] = { 0.28, 0.52, 0.84, 1},	-- Mana
        [1] = { 0.89, 0.18, 0.29, 1}, 	-- Rage
        [2] = {    1, 0.82,    0, 1},	-- Focus
        [3] = {    1, 0.86, 0.10, 1}, 	-- Energy
        [4] = {    0,    1,    1, 1} 	-- Happiness
	},
})

Aloft.Options.args.manaBar =
{
	type = 'group',
	name = L["Mana Bar"],
	desc = L["Mana Bar options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show mana bars on group member nameplates"],
			get = function() return AloftManaBar.db.profile.enable end,
			set = function(v) AloftManaBar.db.profile.enable = v Aloft:DetermineDataSources() AloftManaBar:UpdateAll() end,
			order = 1
		},
		texture =
		{
			type = "text", 
			name = L["Texture"], 
			desc = L["Sets the mana bar texture"],
			get = function() return AloftManaBar.db.profile.texture end,
			set = function(v) AloftManaBar.db.profile.texture = v AloftManaBar:UpdateAll() end,
			disabled = function() return not AloftManaBar.db.profile.enable end,
			validate = SML_statusbar
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust the mana bar's position"],
			disabled = function() return not AloftManaBar.db.profile.enable end,
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the mana bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftManaBar.db.profile.offsets.left end,
					set = function(v) AloftManaBar.db.profile.offsets.left = v AloftManaBar:UpdateAll() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the mana bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftManaBar.db.profile.offsets.right end,
					set = function(v) AloftManaBar.db.profile.offsets.right = v AloftManaBar:UpdateAll() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the mana bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftManaBar.db.profile.offsets.vertical end,
					set = function(v) AloftManaBar.db.profile.offsets.vertical = v AloftManaBar:UpdateAll() end
				},
			},
		},
		height =
		{
			type = 'range',
			name = L["Height"],
			desc = L["Sets the height of the mana bar"],
			min = 2,
			max = 32,
			step = 1,
			disabled = function() return not AloftManaBar.db.profile.enable end,
			get = function() return AloftManaBar.db.profile.height end,
			set = function(v) AloftManaBar.db.profile.height = v AloftManaBar:UpdateAll() end
		},
		colors =
		{
			type = 'group',
			name = L["Colors"],
			desc = L["Customize colors"],
			disabled = function() return not AloftManaBar.db.profile.enable end,
			args =
			{
				backdrop =
				{
					type = 'color',
					name = L["Backdrop Color"],
					desc = L["Sets the backdrop color of the mana bar"],
					get = function() return unpack(AloftManaBar.db.profile.backdropColor) end,
					set = function(r, g, b, a) AloftManaBar.db.profile.backdropColor = { r, g, b, a } AloftManaBar:UpdateAll() end,
					hasAlpha = true,
					order = 1
				},
				mana =
				{
					type = 'color',
					name = L["Mana Color"],
					desc = L["Sets the color for mana bars"],
					get = function() return unpack(AloftManaBar.db.profile.powerTypeColors[0]) end,
					set = function(r, g, b, a) AloftManaBar.db.profile.powerTypeColors[0] = { r, g, b, a } AloftManaBar:UpdateAll() end,
					hasAlpha = true,
					order = 1
				},
				rage =
				{
					type = 'color',
					name = L["Rage Color"],
					desc = L["Sets the color for rage bars"],
					get = function() return unpack(AloftManaBar.db.profile.powerTypeColors[1]) end,
					set = function(r, g, b, a) AloftManaBar.db.profile.powerTypeColors[1] = { r, g, b, a } AloftManaBar:UpdateAll() end,
					hasAlpha = true,
					order = 1
				},
				focus =
				{
					type = 'color',
					name = L["Focus Color"],
					desc = L["Sets the color for focus bars"],
					get = function() return unpack(AloftManaBar.db.profile.powerTypeColors[2]) end,
					set = function(r, g, b, a) AloftManaBar.db.profile.powerTypeColors[2] = { r, g, b, a } AloftManaBar:UpdateAll() end,
					hasAlpha = true,
					order = 1
				},
				energy =
				{
					type = 'color',
					name = L["Energy Color"],
					desc = L["Sets the color for energy bars"],
					get = function() return unpack(AloftManaBar.db.profile.powerTypeColors[3]) end,
					set = function(r, g, b, a) AloftManaBar.db.profile.powerTypeColors[3] = { r, g, b, a } AloftManaBar:UpdateAll() end,
					hasAlpha = true,
					order = 1
				},
				happiness =
				{
					type = 'color',
					name = L["Happiness Color"],
					desc = L["Sets the color for happiness bars"],
					get = function() return unpack(AloftManaBar.db.profile.powerTypeColors[4]) end,
					set = function(r, g, b, a) AloftManaBar.db.profile.powerTypeColors[4] = { r, g, b, a } AloftManaBar:UpdateAll() end,
					hasAlpha = true,
					order = 1
				},
				reset =
				{
					type = "execute",
					name = L["Reset to Defaults"],
					desc = L["Resets all colors to their defaults"],
					order = -1,
					func  = function() 
								AloftManaBar.db.profile.backgroundColor	= { 0.25, 0.25, 0.25, 0.5 }
								AloftManaBar.db.profile.powerTypeColors	=
								{
									[0] = { 0.28, 0.52, 0.84, 1},	-- Mana
									[1] = { 0.89, 0.18, 0.29, 1}, 	-- Rage
									[2] = {    1, 0.82,    0, 1},	-- Focus
									[3] = {    1, 0.86, 0.10, 1}, 	-- Energy
									[4] = {    0,    1,    1, 1} 	-- Happiness
								},
								AloftManaBar:UpdateAll()
							end,
				},
			}
		}
	},
}

-----------------------------------------------------------------------------

local manaBarPool = { }

-----------------------------------------------------------------------------

function AloftManaBar:UpdateAll()
	self:RegisterEvents()
	for aloftData in Aloft:IterateVisibleNameplates() do
		if self.db.profile.enable then
			if aloftData.manaBar then
				self:PlaceBar(aloftData)
				self:SetupBar(aloftData.manaBar)
			end
			self:Update(aloftData)
		else
			self:ReleaseManaBar(aloftData)
		end
	end
	if self.db.profile.enable then
		for manaBar in pairs(manaBarPool) do
			self:SetupBar(manaBar)
		end
	end
end

-----------------------------------------------------------------------------

function AloftManaBar:RegisterEvents()
	self:UnregisterAllEvents()

	if self.db.profile.enable then
		self:RegisterEvent("Aloft:OnNameplateShow", "Update")
		self:RegisterEvent("Aloft:OnManaDataChanged", "Update")
		self:RegisterEvent("Aloft:OnNameplateHide", "ReleaseManaBar")
	end

	self:RegisterEvent("SharedMedia_SetGlobal", function(mediatype, override)
		if mediatype == "statusbar" then
			self:UpdateAll()
		end
	end)
	
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)
end

function AloftManaBar:OnEnable()
	self:UpdateAll()
end

function AloftManaBar:OnDisable()
	self:UnregisterAllEvents()
	
	for aloftData in Aloft:IterateVisibleNameplates() do
		self:ReleaseManaBar(aloftData)
	end
end

function AloftManaBar:RequiresData()
	if self.db.profile.enable then
		return "mana", "maxMana", "powerType"
	end
end

local backdropTable = { }
function AloftManaBar:SetupBar(manaBar)
	local profile = self.db.profile

	local texture = SML:Fetch("statusbar", profile.texture)
	manaBar:SetStatusBarTexture(texture)

	backdropTable.bgFile = texture
	manaBar:SetBackdrop(backdropTable)
	manaBar:SetBackdropColor(unpack(profile.backdropColor))

	-- This manipulates the healthbar background to always display above the frame background
	select(1, manaBar:GetRegions()):SetDrawLayer("OVERLAY")
	select(2, manaBar:GetRegions()):SetDrawLayer("ARTWORK")
end

function AloftManaBar:PlaceBar(aloftData)
	local profile = self.db.profile
	local manaBar = aloftData.manaBar
	manaBar:ClearAllPoints()
	manaBar:SetPoint("TOPLEFT", aloftData.healthBar, "BOTTOMLEFT", profile.offsets.left, profile.offsets.vertical)
	manaBar:SetPoint("BOTTOMRIGHT", aloftData.healthBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical-profile.height)
	manaBar:SetFrameLevel(0)
	manaBar:Show()
end

function AloftManaBar:AcquireManaBar(aloftData)
	local manaBar = aloftData.manaBar
	if not manaBar then
		manaBar = next(manaBarPool)
		if manaBar then
			manaBarPool[manaBar] = nil
			manaBar:SetParent(aloftData.nameplateFrame)
		else
			manaBar = CreateFrame("StatusBar", nil, aloftData.nameplateFrame)
			self:SetupBar(manaBar)
		end
		aloftData.manaBar = manaBar
		self:PlaceBar(aloftData)
	end
end

function AloftManaBar:ReleaseManaBar(aloftData)
	local manaBar = aloftData.manaBar
	if manaBar then
		manaBar:Hide()
		manaBarPool[manaBar] = true
		aloftData.manaBar = nil
	end
end

function AloftManaBar:Update(aloftData)
	if aloftData.mana then
		self:AcquireManaBar(aloftData)
		local color = self.db.profile.powerTypeColors[aloftData.powerType]
		local manaBar = aloftData.manaBar
		manaBar:SetStatusBarColor(unpack(color))
		manaBar:SetMinMaxValues(0, aloftData.maxMana)
		manaBar:SetValue(aloftData.mana)
	else
		self:ReleaseManaBar(aloftData)
	end
end

-----------------------------------------------------------------------------
