-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftManaText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Mana Text"] = true,
	["Mana text options"] = true,
	["Enable"] = true,
	["Show mana text on group member nameplates"] = true,

	["Typeface"] = true,
	["Mana text typeface options"] = true,
	["Font"] = true,
	["Sets the font for mana text"] = true,
	["Font Size"] = true,
	["Sets the font height of the mana text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on mana text"] = true,
	["Outline"] = true,
	["Sets the outline for mana text"] = true,
	["Mode"] = true,
	["Choose what to show for mana text"] = true,

	["Position"] = true,
	["Adjust mana text position"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the mana text"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the mana text"] = true,
	["X Offset"] = true,
	["X offset of the mana text"] = true,
	["Y Offset"] = true,
	["Y offset of the mana text"] = true,
	["Alpha"] = true,
	["Sets the alpha of the mana text"] = true,
	["Color"] = true,
	["Sets the mana text color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
	
	["Percent"] = true,
	["Deficit"] = true,
	["Mana"] = true,
	["Full"] = true,

	["Advanced"] = true,
	["Options for expert users"] = true,
	["Format"] = true,
	["Mana tag"] = true,
	["<Any tag string>"] = true,
} end)

-----------------------------------------------------------------------------

local manaTags =
{
	NONE				= "",
	PERCENT				= "[manaPercent:percent]",
	MANA				= "[mana:short]",
	DEFICIT				= "[manaDeficit:hideZero:short]",
	FULL				= "[mana:short:suffix(\" (\")][manaPercent:percent:suffix(\")\")]",
}

-----------------------------------------------------------------------------

local AloftManaText = Aloft:NewModule("ManaText", "AceEvent-2.0")
AloftManaText.db = Aloft:AcquireDBNamespace("manaText")
Aloft:RegisterDefaults("manaText", "profile", {
	enable			= false,
	alpha			= 1,
	point			= "RIGHT",
	relativeToPoint	= "RIGHT",
	offsetX			= 0,
	offsetY			= 0,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= false,
	outline			= "OUTLINE",
	mode			= "PERCENT",
	format			= "",
	color			= { 1, 1, 1 }
})

Aloft.Options.args.manaBar.args.manaText =
{
	type = 'group',
	name = L["Mana Text"],
	desc = L["Mana text options"],
	disabled = function() return not Aloft:IsActive() end,
	order = -1,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show mana text on group member nameplates"],
			get = function() return AloftManaText.db.profile.enable end,
			set = function(v) AloftManaText.db.profile.enable = v AloftManaText:UpdateTag() AloftManaText:UpdateAll() end,
			order = 1
		},
		typeface =
		{
			type = "group",
			name = L["Typeface"],
			desc = L["Mana text typeface options"],
			disabled = function() return not AloftManaText.db.profile.enable end,
			args =
			{
				font =
				{
					type = "text", 
					name = L["Font"], 
					desc = L["Sets the font for mana text"],
					get = function() return AloftManaText.db.profile.font end,
					set = function(v) AloftManaText.db.profile.font = v AloftManaText:UpdateAll() end,
					validate = SML_font,
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the mana text"],
					max = 16,
					min = 5,
					step = 1,
					get = function() return AloftManaText.db.profile.fontSize end,
					set = function(value) AloftManaText.db.profile.fontSize = value AloftManaText:UpdateAll() end
				},
				shadow =
				{
					type = 'toggle',
					name = L["Font Shadow"],
					desc = L["Show font shadow on mana text"],
					get = function() return AloftManaText.db.profile.shadow end,
					set = function(v) AloftManaText.db.profile.shadow = v AloftManaText:UpdateAll() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the outline for mana text"],
					get = function() return AloftManaText.db.profile.outline end,
					set = function(value) AloftManaText.db.profile.outline = value AloftManaText:UpdateAll() end,
					validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
				},
			},
		},
		mode =
		{
			type = 'text',
			name = L["Mode"],
			desc = L["Choose what to show for mana text"],
			disabled = function() return not AloftManaText.db.profile.enable end,
			get = function() return AloftManaText.db.profile.mode end,
			set = function(value) AloftManaText.db.profile.mode = value AloftManaText:UpdateTag() AloftManaText:UpdateAll() end,
			validate = { PERCENT = L["Percent"], DEFICIT = L["Deficit"], MANA = L["Mana"], FULL = L["Full"] },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust mana text position"],
			disabled = function() return not AloftManaText.db.profile.enable end,
			args =
			{
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the mana text"],
					get = function() return AloftManaText.db.profile.point end,
					set = function(v) AloftManaText.db.profile.point = v AloftManaText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the mana text"],
					get = function() return AloftManaText.db.profile.relativeToPoint end,
					set = function(v) AloftManaText.db.profile.relativeToPoint = v AloftManaText:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the mana text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftManaText.db.profile.offsetX end,
					set = function(v) AloftManaText.db.profile.offsetX = v AloftManaText:UpdateAll() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the mana text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftManaText.db.profile.offsetY end,
					set = function(v) AloftManaText.db.profile.offsetY = v AloftManaText:UpdateAll() end
				},
			},
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the alpha of the mana text"],
			min = 0,
			max = 1.0,
			step = 0.05,
			disabled = function() return not AloftManaText.db.profile.enable end,
			get = function() return AloftManaText.db.profile.alpha end,
			set = function(v) AloftManaText.db.profile.alpha = v AloftManaText:UpdateAll() end
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the mana text color"],
			disabled = function() return not AloftManaText.db.profile.enable end,
			get = function() return unpack(AloftManaText.db.profile.color) end,
			set = function(r, g, b, a) AloftManaText.db.profile.color = { r, g, b, a } AloftManaText:UpdateAll() end,
		},
		advanced  =
		{
			type = 'group',
			name = L["Advanced"],
			desc = L["Options for expert users"],
			order = -1,
			disabled = function() return not AloftManaText.db.profile.enable end,
			args =
			{
				format = 
				{
					type = 'text',
					name = L["Format"],
					desc = L["Mana tag"],
					usage = L["<Any tag string>"],
					get = function() return AloftManaText.db.profile.format end,
					set = function(v) AloftManaText.db.profile.format = v Aloft:DetermineDataSources() AloftManaText:UpdateAll() end,
				}
			},
		},
	},
}

-----------------------------------------------------------------------------

local textMethod = nil
local textMethodData = { }
local textRegionPool = { }

-----------------------------------------------------------------------------

function AloftManaText:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		if self.db.profile.enable then
			if aloftData.manaTextRegion then
				self:PrepareText(aloftData.manaTextRegion, self.db.profile)
				self:PlaceFrame(aloftData.manaTextRegion, aloftData.healthBar, self.db.profile)
			end
			if aloftData:IsShown() then
				self:UpdateText(aloftData)
			end
		else
			self:OnNameplateHide(aloftData)
		end
	end
	for textRegion in pairs(textRegionPool) do
		self:PrepareText(textRegion, self.db.profile)
	end
end

-----------------------------------------------------------------------------

function AloftManaText:UpdateTag()
	local tag = self.db.profile.mode
	if not self.db.profile.enable then tag = "NONE"	end
	self.db.profile.format = manaTags[tag]

	Aloft:DetermineDataSources()
end

-----------------------------------------------------------------------------

function AloftManaText:RegisterEvents()
	local profile = self.db.profile
	self:UnregisterAllEvents()

	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)
	
	if profile.enable then
		self:RegisterEvent("Aloft:OnNameplateShow", "UpdateText")
		self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")

		for events in pairs(textMethodData.events) do
			self:RegisterEvent(events, "UpdateText")
		end
	end
end

local dataRequiredList = { }
function AloftManaText:RequiresData()
	local profile = self.db.profile

	textMethodData = AloftTag:CreateTag(profile.format)
	textMethod = textMethodData.method

	self:RegisterEvents()

	if not profile.enable then
		return
	end

	for i=1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	for data in pairs(textMethodData.data) do
		table.insert(dataRequiredList, data)
	end

	return unpack(dataRequiredList)
end

function AloftManaText:OnEnable()
	self:UpdateAll()
end

function AloftManaText:OnDisable()
	self:UnregisterAllEvents()
	
	for aloftData in Aloft:IterateVisibleNameplates() do
		self:OnNameplateHide(aloftData)
	end
end

function AloftManaText:UpdateText(aloftData)
    local text = textMethod(aloftData)
            
    if text ~= "" then
        -- Check if this nameplate already has a text region assigned to it
        local manaTextRegion = aloftData.manaTextRegion
        if not manaTextRegion then
            manaTextRegion = next(textRegionPool)
            if manaTextRegion then
                -- We have a recycled text region. Text format already setup, just need to place it
                textRegionPool[manaTextRegion] = nil
                manaTextRegion:SetParent(aloftData.nameplateFrame)
                manaTextRegion:Show()
            else
                -- We need a new text region.
                manaTextRegion = aloftData:CreateFontString()
                self:PrepareText(manaTextRegion, self.db.profile)
            end
            self:PlaceFrame(manaTextRegion, aloftData.healthBar, self.db.profile)
            aloftData.manaTextRegion = manaTextRegion
        end
        manaTextRegion:SetText(text)
    else
        self:OnNameplateHide(aloftData)
    end
end

function AloftManaText:OnNameplateHide(aloftData)
    local manaTextRegion = aloftData.manaTextRegion
    if manaTextRegion then
        manaTextRegion:Hide()
        textRegionPool[manaTextRegion] = true
        aloftData.manaTextRegion = nil
    end
end

-----------------------------------------------------------------------------
