-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftSpellNameText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Spell Name"] = true,
	["Cast bar spell name text options"] = true,
	["Enable"] = true,
	["Show spell name on cast bar"] = true,
	["Typeface"] = true,
	["Spell name typeface options"] = true,
	["Font"] = true, 
	["Sets the font for spell name"] = true,
	["Font Size"] = true,
	["Sets the font height of the spell name"] = true,
	["Font Shadow"] = true,
	["Show font shadow on spell name"] = true,
	["Alignment"] = true,
	["Sets the alignment of the spell name"] = true,
	["Outline"] = true,
	["Sets the font outline for spell name"] = true,
	["Position"] = true,
	["Adjust spell name position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the text"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the text"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the text"] = true,
	["Color"] = true,
	["Sets the spell name color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
} end)

-----------------------------------------------------------------------------

local spellNameTextRegion = nil

-----------------------------------------------------------------------------

local AloftSpellNameText = Aloft:NewModule("SpellNameText", "AceEvent-2.0")
AloftSpellNameText.db = Aloft:AcquireDBNamespace("spellNameText")
Aloft:RegisterDefaults("spellNameText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= true,
	alignment		= "LEFT",
	outline			= "",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
	color			= { 1, 1, 1, 1 }
})

Aloft.Options.args.castBar.args.spellNameText =
{
	type = 'group',
	name = L["Spell Name"],
	desc = L["Cast bar spell name text options"],
	disabled = function() return not Aloft:IsActive() or GetCVar("ShowVKeyCastbar") ~= "1" end,
	order = -1,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show spell name on cast bar"],
			get = function() return AloftSpellNameText.db.profile.enable end,
			set = function(v) AloftSpellNameText.db.profile.enable = v AloftSpellNameText:Update() end,
			order = 1
		},
		typeface =
		{
			type = "group",
			name = L["Typeface"],
			desc = L["Spell name typeface options"],
			disabled = function() return not AloftSpellNameText.db.profile.enable end,
			args =
			{
				font =
				{
					type = "text", 
					name = L["Font"], 
					desc = L["Sets the font for spell name"],
					get = function() return AloftSpellNameText.db.profile.font end,
					set = function(v) AloftSpellNameText.db.profile.font = v AloftSpellNameText:Update() end,
					validate = SML_font,
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the spell name"],
					max = 16,
					min = 5,
					step = 1,
					get = function() return AloftSpellNameText.db.profile.fontSize end,
					set = function(value) AloftSpellNameText.db.profile.fontSize = value AloftSpellNameText:Update() end
				},
				shadow =
				{
					type = 'toggle',
					name = L["Font Shadow"],
					desc = L["Show font shadow on spell name"],
					get = function() return AloftSpellNameText.db.profile.shadow end,
					set = function(v) AloftSpellNameText.db.profile.shadow = v AloftSpellNameText:Update() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the font outline for spell name"],
					get = function() return AloftSpellNameText.db.profile.outline end,
					set = function(value) AloftSpellNameText.db.profile.outline = value AloftSpellNameText:Update() end,
					validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
				},
			},
		},
		alignment =
		{
			type = 'text',
			name = L["Alignment"],
			desc = L["Sets the alignment of the spell name"],
			disabled = function() return not AloftSpellNameText.db.profile.enable end,
			get = function() return AloftSpellNameText.db.profile.alignment end,
			set = function(value) AloftSpellNameText.db.profile.alignment = value AloftSpellNameText:Update() end,
			validate = {"LEFT", "CENTER", "RIGHT" },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust spell name position"],
			disabled = function() return not AloftSpellNameText.db.profile.enable end,
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftSpellNameText.db.profile.offsets.left end,
					set = function(v) AloftSpellNameText.db.profile.offsets.left = v AloftSpellNameText:Update() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftSpellNameText.db.profile.offsets.right end,
					set = function(v) AloftSpellNameText.db.profile.offsets.right = v AloftSpellNameText:Update() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftSpellNameText.db.profile.offsets.vertical end,
					set = function(v) AloftSpellNameText.db.profile.offsets.vertical = v AloftSpellNameText:Update() end
				},
			},
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the spell name color"],
			disabled = function() return not AloftSpellNameText.db.profile.enable end,
			get = function() return unpack(AloftSpellNameText.db.profile.color) end,
			set = function(r, g, b, a) AloftSpellNameText.db.profile.color = { r, g, b, a } AloftSpellNameText:Update() end,
			hasAlpha = true
		},
	},
}

-----------------------------------------------------------------------------

function AloftSpellNameText:Update()
	self:CreateFrame()
	self:RegisterEvents()
	
	if self.db.profile.enable then
		for aloftData in Aloft:IterateVisibleNameplates() do
			self:SetupFrame(aloftData)
		end
	end
end

-----------------------------------------------------------------------------

function AloftSpellNameText:CreateFrame()
	if not self.db.profile.enable then
		if spellNameTextRegion then
			spellNameTextRegion:Hide()
		end
		return
	end

	if not spellNameTextRegion then
		spellNameTextRegion = UIParent:CreateFontString()
	end
	
	self:PrepareText(spellNameTextRegion, self.db.profile)

end

function AloftSpellNameText:RegisterEvents()
	if self.db.profile.enable then	
		self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
		self:RegisterEvent("Aloft:OnCastBarValueChanged", "UpdateText")
		self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
		self:RegisterEvent("Aloft:OnCastBarHide", "OnCastBarHide")
	else
		self:CancelEvent("Aloft:SetupFrame")
		self:CancelEvent("Aloft:OnCastBarValueChanged")
		self:CancelEvent("Aloft:OnCastBarShow")
		self:CancelEvent("Aloft:OnCastBarHide")
	end
end

-----------------------------------------------------------------------------

function AloftSpellNameText:GetTargetSpellName()
	local _, _, displayName = UnitCastingInfo("target")
	if displayName then
		return displayName
	end
	_, _, displayName = UnitChannelInfo("target")
	if displayName then
		return displayName
	end
	return ""
end

-----------------------------------------------------------------------------

function AloftSpellNameText:OnEnable()
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:Update()
		end
	end)
	
	self:Update()
end

function AloftSpellNameText:OnDisable()
	self:UnregisterAllEvents()
	
	if spellNameTextRegion then
		spellNameTextRegion:Hide()
	end
end

function AloftSpellNameText:SetupFrame(aloftData)
	if aloftData:IsCurrentTarget() then
		self:OnCastBarShow(aloftData)
	end	
end

function AloftSpellNameText:UpdateText(aloftData)
	local castBar = aloftData.castBar
	
	if aloftData.castBarStartTime ~= castBar:GetMinMaxValues() then
		spellNameTextRegion:SetText(self:GetTargetSpellName())
	end
end

function AloftSpellNameText:OnCastBarShow(aloftData)
	local castBar = aloftData.castBar
	local profile = self.db.profile

	spellNameTextRegion:SetText(self:GetTargetSpellName())
	spellNameTextRegion:ClearAllPoints()
	spellNameTextRegion:SetPoint("TOPLEFT", castBar, "TOPLEFT", profile.offsets.left, profile.offsets.vertical)
	spellNameTextRegion:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical)
	spellNameTextRegion:SetAlpha(1.0)
	spellNameTextRegion:SetParent(aloftData.nameplateFrame)
	spellNameTextRegion:Show()

	aloftData.castBarStartTime = castBar:GetMinMaxValues()

	self:ScheduleRepeatingEvent("AloftSpellNameText:UpdateAlpha", self.UpdateAlpha, 0.1, self, aloftData.castBar)
end

function AloftSpellNameText:OnCastBarHide(aloftData)
	self:CancelScheduledEvent("AloftSpellNameText:UpdateAlpha")
	spellNameTextRegion:Hide()
end

function AloftSpellNameText:UpdateAlpha(castBar)
	spellNameTextRegion:SetAlpha(select(4, castBar:GetStatusBarColor()))
end

-----------------------------------------------------------------------------
